import json

import tornado.web

from rest_framework.filter import BaseFilter
from rest_framework.model import BaseModel


class BaseHandler(tornado.web.RequestHandler):

    def initialize(self) -> None:
        self.query_condition = self.parse_query_arguments()  # 查询条件
        self.update_condition = self.parse_body_arguments()  #

    def parse_query_arguments(self):
        query_condition = {}
        filter_class = self.get_filter_class()
        if not filter_class:
            return
        for key in filter_class.__annotations__.keys():
            if self.request.query_arguments.__contains__(key):
                query_condition[key] = self.request.query_arguments.get(key)[0].decode()
        return query_condition

    def parse_body_arguments(self):
        update_condition = {}
        serializer_class = self.get_serializer_class()
        for key in serializer_class.Meta.fields:
            if self.request.body_arguments.__contains__(key):
                update_condition[key] = self.request.body_arguments.get(key)[0].decode()
        return update_condition

    def get_update_condition(self):
        return getattr(self, "update_condition", None)

    def get_query_condition(self):
        return getattr(self, "query_condition", None)

    def get_query_set(self):
        return getattr(self, "query_set", None)

    def get_model_class(self):
        model_class = getattr(self, "model_class", None)
        assert issubclass(model_class, BaseModel)
        return model_class

    def get_serializer_class(self):
        return getattr(self, "serializer_class", None)

    def get_filter_class(self):
        if (filter_class := getattr(self, "filter_class", None)) is None:
            return
        assert issubclass(filter_class, BaseFilter)
        return filter_class


class ListHandler(BaseHandler):

    async def get(self, *args, **kwargs):
        model_class = self.get_model_class()
        model = model_class().filter(self.get_query_condition())
        self.write(json.dumps(model, ensure_ascii=False))

    async def post(self, *args, **kwargs):
        model_class = self.get_model_class()
        model = model_class().create(self.get_update_condition())
        self.write(json.dumps(model, ensure_ascii=False))


class DetailHandler(BaseHandler):

    async def get(self, **kwargs):
        model_class = self.get_model_class()
        model = model_class().get(kwargs["id"])
        self.write(json.dumps(model, ensure_ascii=False))

    async def put(self, **kwargs):
        model_class = self.get_model_class()
        model = model_class().update(kwargs["id"], self.get_update_condition())
        self.write(json.dumps(model, ensure_ascii=False))

    async def patch(self, **kwargs):
        model_class = self.get_model_class()
        model = model_class().update(kwargs["id"], self.get_update_condition(), partial=True)
        self.write(json.dumps(model, ensure_ascii=False))

    async def delete(self, **kwargs):
        model_class = self.get_model_class()
        model = model_class().delete(kwargs["id"])
        self.write(json.dumps(model, ensure_ascii=False))



class ModelNoneError(Exception):
    def __init__(self, *args, **kwargs):
        pass


class SerializerNoneError(Exception):
    def __init__(self, *args, **kwargs):
        pass


class FilterNoneError(Exception):
    def __init__(self, *args, **kwargs):
        pass



#!/usr/bin/env python
#
# Copyright 2009 Facebook
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""A lightweight wrapper around MySQLdb.

Originally part of the Tornado framework.  The tornado.database module
is slated for removal in Tornado 3.0, and it is now available separately
as torndb.
"""

# 在torndb基础上进行python3适应修改，原tornado地址：https://github.com/bdarnell/torndb

from __future__ import absolute_import, division, with_statement

import copy
import itertools
import logging
import os
import time

try:
    import MySQLdb.constants
    import MySQLdb.converters
    import MySQLdb.cursors
except ImportError:
    # If MySQLdb isn't available this module won't actually be useable,
    # but we want it to at least be importable on readthedocs.org,
    # which has limitations on third-party modules.
    if 'READTHEDOCS' in os.environ:
        MySQLdb = None
    else:
        raise

version = "0.3"
version_info = (0, 3, 0, 0)


class Connection(object):
    """A lightweight wrapper around MySQLdb DB-API connections.

    The main value we provide is wrapping rows in a dict/object so that
    columns can be accessed by name. Typical usage::

        db = torndb.Connection("localhost", "my_database")
        for article in db.query("SELECT * FROM articles"):
            print article.title

    Cursors are hidden by the implementation, but other than that, the methods
    are very similar to the DB-API.

    We explicitly set the timezone to UTC and assume the character encoding to
    UTF-8 (can be changed) on all connections to avoid time zone and encoding errors.

    The sql_mode parameter is set by default to "traditional", which "gives an error instead of a warning"
    (http://dev.mysql.com/doc/refman/5.0/en/server-sql-mode.html). However, it can be set to
    any other mode including blank (None) thereby explicitly clearing the SQL mode.
    """

    def __init__(self, host, database, user=None, password=None,
                 max_idle_time=7 * 3600, connect_timeout=10,
                 time_zone="+0:00", charset="utf8", sql_mode="TRADITIONAL"):
        self.host = host
        self.database = database
        self.max_idle_time = float(max_idle_time)

        args = dict(conv=CONVERSIONS, use_unicode=True, charset=charset,
                    db=database, init_command=('SET time_zone = "%s"' % time_zone),
                    connect_timeout=connect_timeout, sql_mode=sql_mode)
        if user is not None:
            args["user"] = user
        if password is not None:
            args["passwd"] = password

        # We accept a path to a MySQL socket file or a host(:port) string
        if "/" in host:
            args["unix_socket"] = host
        else:
            self.socket = None
            pair = host.split(":")
            if len(pair) == 2:
                args["host"] = pair[0]
                args["port"] = int(pair[1])
            else:
                args["host"] = host
                args["port"] = 3306

        self._db = None
        self._db_args = args
        self._last_use_time = time.time()
        try:
            self.reconnect()
        except Exception:
            logging.error("Cannot connect to MySQL on %s", self.host,
                          exc_info=True)

    def __del__(self):
        self.close()

    def close(self):
        """Closes this database connection."""
        if getattr(self, "_db", None) is not None:
            self._db.close()
            self._db = None

    def reconnect(self):
        """Closes the existing database connection and re-opens it."""
        self.close()
        self._db = MySQLdb.connect(**self._db_args)
        self._db.autocommit(True)

    def iter(self, query, *parameters, **kwparameters):
        """Returns an iterator for the given query and parameters."""
        self._ensure_connected()
        cursor = MySQLdb.cursors.SSCursor(self._db)
        try:
            self._execute(cursor, query, parameters, kwparameters)
            column_names = [d[0] for d in cursor.description]
            for row in cursor:
                yield Row(zip(column_names, row))
        finally:
            cursor.close()

    def query(self, query, *parameters, **kwparameters):
        """Returns a row list for the given query and parameters."""
        cursor = self._cursor()
        try:
            self._execute(cursor, query, parameters, kwparameters)
            column_names = [d[0] for d in cursor.description]
            return [Row(itertools.zip_longest(column_names, row)) for row in cursor]
        finally:
            cursor.close()

    def get(self, query, *parameters, **kwparameters):
        """Returns the (singular) row returned by the given query.

        If the query has no results, returns None.  If it has
        more than one result, raises an exception.
        """
        rows = self.query(query, *parameters, **kwparameters)
        if not rows:
            return None
        elif len(rows) > 1:
            raise Exception("Multiple rows returned for Database.get() query")
        else:
            return rows[0]

    # rowcount is a more reasonable default return value than lastrowid,
    # but for historical compatibility execute() must return lastrowid.
    def execute(self, query, *parameters, **kwparameters):
        """Executes the given query, returning the lastrowid from the query."""
        return self.execute_lastrowid(query, *parameters, **kwparameters)

    def execute_lastrowid(self, query, *parameters, **kwparameters):
        """Executes the given query, returning the lastrowid from the query."""
        cursor = self._cursor()
        try:
            self._execute(cursor, query, parameters, kwparameters)
            return cursor.lastrowid
        finally:
            cursor.close()

    def execute_rowcount(self, query, *parameters, **kwparameters):
        """Executes the given query, returning the rowcount from the query."""
        cursor = self._cursor()
        try:
            self._execute(cursor, query, parameters, kwparameters)
            return cursor.rowcount
        finally:
            cursor.close()

    def executemany(self, query, parameters):
        """Executes the given query against all the given param sequences.

        We return the lastrowid from the query.
        """
        return self.executemany_lastrowid(query, parameters)

    def executemany_lastrowid(self, query, parameters):
        """Executes the given query against all the given param sequences.

        We return the lastrowid from the query.
        """
        cursor = self._cursor()
        try:
            cursor.executemany(query, parameters)
            return cursor.lastrowid
        finally:
            cursor.close()

    def executemany_rowcount(self, query, parameters):
        """Executes the given query against all the given param sequences.

        We return the rowcount from the query.
        """
        cursor = self._cursor()
        try:
            cursor.executemany(query, parameters)
            return cursor.rowcount
        finally:
            cursor.close()

    update = execute_rowcount
    updatemany = executemany_rowcount

    insert = execute_lastrowid
    insertmany = executemany_lastrowid

    def _ensure_connected(self):
        # Mysql by default closes client connections that are idle for
        # 8 hours, but the client library does not report this fact until
        # you try to perform a query and it fails.  Protect against this
        # case by preemptively closing and reopening the connection
        # if it has been idle for too long (7 hours by default).
        if (self._db is None or
                (time.time() - self._last_use_time > self.max_idle_time)):
            self.reconnect()
        self._last_use_time = time.time()

    def _cursor(self):
        self._ensure_connected()
        return self._db.cursor()

    def _execute(self, cursor, query, parameters, kwparameters):
        try:
            return cursor.execute(query, kwparameters or parameters)
        except OperationalError:
            logging.error("Error connecting to MySQL on %s", self.host)
            self.close()
            raise


class Row(dict):
    """A dict that allows for object-like property access syntax."""

    def __getattr__(self, name):
        try:
            return self[name]
        except KeyError:
            raise AttributeError(name)


if MySQLdb is not None:
    # Fix the access conversions to properly recognize unicode/binary
    FIELD_TYPE = MySQLdb.constants.FIELD_TYPE
    FLAG = MySQLdb.constants.FLAG
    CONVERSIONS = copy.copy(MySQLdb.converters.conversions)

    field_types = [FIELD_TYPE.BLOB, FIELD_TYPE.STRING, FIELD_TYPE.VAR_STRING]
    if 'VARCHAR' in vars(FIELD_TYPE):
        field_types.append(FIELD_TYPE.VARCHAR)

    for field_type in field_types:
        CONVERSIONS[field_type] = [(FLAG.BINARY, str)].append(CONVERSIONS[field_type])

    # Alias some common MySQL exceptions
    IntegrityError = MySQLdb.IntegrityError
    OperationalError = MySQLdb.OperationalError



import asyncio
import time


async def test_coroutine_1():
    time.sleep(1)
    return "test_coroutine_1"


async def test_coroutine_2():
    await asyncio.sleep(1)
    return "test_coroutine_2"


if __name__ == '__main__':

    print("start...")
    start = time.time()
    test1 = test_coroutine_1()
    try:
        test1.send(None)
    except StopIteration as ex:
        pass
        # print(ex)
    print(f"done test1 used {time.time() - start}")

    print()

    start = time.time()
    test2 = test_coroutine_2()
    res = asyncio.run(test2)
    # print(res)

    print(f"done test2, used {time.time() - start}")



from rest_framework.conf import config
from rest_framework.torndb import Connection


class BaseModel(object):
    """"""

    def __init__(self):
        self.db = Connection(**config["mysql"])

    def filter(self, query_condition):
        """过滤"""
        assert isinstance(query_condition, dict)
        parameters = [f'%{value}%' for value in query_condition.values()]
        where = " AND ".join([f"{key} LIKE %s" for key in query_condition.keys()])
        where = f"WHERE {where}" if where else ""
        query = f"SELECT * FROM {self.Meta.table_name} {where}"
        return self.db.query(query, *parameters)

    def get(self, pk):
        """获取单个"""
        query = f"SELECT * FROM {self.Meta.table_name} WHERE `id` = %s"
        return self.db.get(query, pk)

    def create(self, data):
        """新增"""
        assert isinstance(data, dict)
        fields = ", ".join(data.keys())
        values = ", ".join([f"'{item}'" for item in data.values()])
        sql = f"INSERT INTO {self.Meta.table_name} ({fields}) VALUES ({values})"
        return self.db.execute(sql)

    def update(self, pk, data, partial=False):
        """更新"""
        return self.partial_update(pk, data)

    def partial_update(self, pk, data):
        """部分更新"""
        assert isinstance(data, dict)
        update_condition = ", ".join([f"{key}='{value}'" for key, value in data.items()])
        if not update_condition:
            return
        sql = f"UPDATE {self.Meta.table_name} SET {update_condition} WHERE `id` = {pk}"
        print(sql)
        return self.db.execute(sql)

    def delete(self, pk):
        """删除"""
        query = f"DELETE FROM {self.Meta.table_name} WHERE id = %s"
        return self.db.execute(query, pk)



import asyncio

import _asyncio


async def fun1():
    print("start fun1")
    await asyncio.sleep(1)
    print("done fun1")
    return "fun1"


async def fun2():
    print("start fun2")
    await fun1()
    print("done fun2")
    return "fun2"


task_list = [fun1(), fun2()]

done, pending = asyncio.run(asyncio.wait(task_list))

# print(done)
# print(pending)

assert isinstance(done, set)
res = done.pop()
assert isinstance(res, _asyncio.Task)
print(res.result())
print(res.get_name())
print(res.get_coro())
print(res.get_stack())
print(res)



from tornado.web import RequestHandler


class Response:
    def __init__(self, message):
        self.message = message

    def __str__(self):
        return f"{self.__class__} message: {self.message}"


if __name__ == '__main__':
    response = Response("hello")
    print(response)



import os
import json

ROOT_PATH = os.getenv("TORNADO_WORKDIR", os.getcwd())
config = {}
config_path = os.path.join(ROOT_PATH, "config.json")
if os.path.isfile(config_path):
    with open(config_path, "r", encoding="utf8") as fp:
        config = json.load(fp)
__all__ = [
    config
]

if __name__ == '__main__':
    print(config)
    print(ROOT_PATH)
